/* System */
#define _POSIX_SOURCE 1
#include <pthread.h>
#include <stdio.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <limits.h>
#include <string.h>
#include <gtk/gtk.h>
#include <sys/select.h>




/* User */
#include "socket.h"
#include "commons.h"
#include "gui.h"
#include "client.h"
#include "command.h"
#include "listener.h"
#include "live.h"
#include "../common/messages.h"
#include "../common/network.h"
#include "../common/networktransfer.h"


/* globale Funktionen */
static void printHelp();
/* globale Variable */
static char toggleRequestWrite = 1;
char *app_name;
struct table_status stat;



 /**************************************************************************************\
 * Funktion:            updateGUIstate                                                  *
 *                                                                                      *
 * Kurzbeschreibung:  	Helperfunktion                                                  *
 *                      Aktualisiert die Widgets der Gui in Abhängigkeit der globalen	*
 *                      Variablen.                                                      *
 *                                                                                      *
 * Ergebnis:            int: Erfolgsstatus                                              *
 *                                                                                      *
 * Import-Parameter:    myRole, myName, myClientID                                      *
 *                      currentBoardMode                                                *
 *                      haveWritePermission                                             *
 *                      currentBoardTimestamp                                           *
 *                                                                                      *
 * Export-Parameter:    -                                                               *
 *                                                                                      *
 \**************************************************************************************/
int updateGUIstate(void)
{
	static char tmp[256];
	GdkColor tmpColor;

/*	gdk_color_parse ("#385030", &tmpColor); */
	gdk_color_parse ("#386038", &tmpColor); 
	gtk_widget_modify_base(GTK_WIDGET(textview), GTK_STATE_NORMAL, &tmpColor); /* Tafelhintergrundfarbe setzen */

	/* Statusleiste */
	gtk_label_set_text(statusWrite, "Tafelrechte\n[lesen/schreiben]");

	sprintf(tmp, "Name: \t[%s]\nRolle: \t[%s]\nClientID:\t[%i]",
			"Test", 1?"Dozent":(0?"Tutor":"Student"), 1);
	gtk_label_set_text(statusRole, tmp);

	sprintf(tmp, "Zeit der letzten Änderung\n[%i]", 12345);
	gtk_label_set_text(statusVersion, tmp);

	sprintf(tmp, "Dozenten: \t[%i]\nTutoren:    \t[%i]\nStudenten: \t[%i]", 1, 2, 3);
	gtk_label_set_text(statusUsers, tmp);

	/* Buttons */
	sprintf(tmp,"Schreibrecht %s", toggleRequestWrite?"anfragen":"abgeben" );
	gtk_button_set_label(GTK_BUTTON(requestWrite),tmp);

	return 0;
}


 /**************************************************************************************\
 * Funktion:            on_application_exit                                             *
 *                                                                                      *
 * Kurzbeschreibung:    Callback                                                        *
 *                      Wird beim betätigen des X-Buttons(oben rechts) aufgerufen.      *
 *                      Beendet eventuell (Benutzerentscheidung) den Client             *
 *                                                                                      *
 * Ergebnis:            gint: FALSE "destroy" einleiten; TRUE nichts machen             *
 *                                                                                      *
 * Import-Parameter:    widget: Referenz auf das auslösende Widget                      *
 *                      event: 	Eventreferenz                                           *
 *                      daten:  Zusatzinformationen                                     *
 *                                                                                      *
 * Export-Parameter:    -                                                               *
 *                                                                                      *
 \**************************************************************************************/
gint on_application_exit(GtkWidget * widget, GdkEvent event, gpointer daten)
{
	if(popupQuestionDialog("Sind Sie sicher?","Wollen Sie das Programm wirklich beenden?"))
	{
		gtk_main_quit();
		return FALSE;  /* Event "destroy" einleiten */
	}
	return TRUE;
}


 /**************************************************************************************\
 * Funktion:                    on_button_request_write_clicked                         *
 \**************************************************************************************/
void on_button_request_write_clicked (GtkButton * button, gpointer user_data)
{
	toggleRequestWrite = toggleRequestWrite?0:1;
	updateGUIstate();
}


 /**************************************************************************************\
 * Funktion:                    on_button_quit_all_clicked                              *
 \**************************************************************************************/
void on_button_quit_all_clicked (GtkButton * button, gpointer user_data)
{
	if(popupQuestionDialog("Sind Sie sicher?","Wollen Sie das Programm wirklich beenden?"))
		gtk_main_quit();
}


 /**************************************************************************************\
 * Funktion:                    on_button_quit_clicked                                  *
 \**************************************************************************************/
void on_button_quit_clicked ( GtkButton * button, gpointer user_data )
{
	if(popupQuestionDialog("Sind Sie sicher?","Wollen Sie das Programm wirklich beenden?"))
		gtk_main_quit();
}

 /**************************************************************************************\
 * Funktion:                    on_button_log_clicked                                   *
 \**************************************************************************************/
void on_button_log_clicked ( GtkButton * button, gpointer user_data )
{
}


 /**************************************************************************************\
 * Funktion:          on_button_change_mode_clicked                                     *
 \**************************************************************************************/
void on_button_remove_permissions_clicked ( GtkButton * button, gpointer user_data )
{
}


 /**************************************************************************************\
 * Funktion:           on_text_buffer_changed                                           *
 \**************************************************************************************/
void on_text_buffer_changed (GtkTextBuffer *textbuffer, gpointer user_data)
{
	/* popupMessage("HALLO!!!"); */
}





 /**************************************************************************************\
 * Funktion: main                                                                       *
 \**************************************************************************************/
int main (int argc, char **argv)

{
	pthread_t com_thread, list_thread, live_thread;
	pthread_mutex_t socket_mutex = PTHREAD_MUTEX_INITIALIZER;
	char *port, *servername, *user;
	int socket;
	int ret = -1; /*return value*/
	int i;	/*Temp value*/
	fd_set socket_fd;


	/** CLI Parameter auswerten **/

	/* Default Settings*/
	port = "50000";
	user = "asdfghffffff";
	app_name = "app-name";
	stat.role = 4;
	stat.name = app_name;
	servername = "127.0.0.1";
	
	if (argc<2){
		printHelp();
		return EXIT_SUCCESS;
	}

	app_name = argv[0];


	/*CLI Command for help*/

	if (!strcmp("-?", argv[1])|| !strcmp("--help", argv[1])){
		printHelp();
		return EXIT_SUCCESS;
	}

	for (i=1;i<argc; i++){

		/*Abfrage auf Portnummer*/
		if (!strcmp ("-p",argv[i])){
			if (i+1<argc){
			port = argv[i+1];
			i = i+1;
			printf("Port: %s \n", port);
			}
			else{
				printHelp();
				return EXIT_FAILURE;
			}
		}
		/*Abfrage auf Nutzernamen*/
		else if (!strcmp ("-n", argv[i])){
			if(i+1<argc){
			user = argv[i+1];
			i++;
			printf("Benutzername: %s \n", user);
			}
			else{
				printHelp();
				return EXIT_FAILURE;
			}
		}


		/* Abfrage auf Rolle des Benutzers */
		else if (!strcmp ("-s", argv[i])){
			stat.role = RFC_ROLLE_STUD;
		}
		else if (!strcmp ("-d", argv[i])){
			stat.role = RFC_ROLLE_DOZ;
		}
		else if (!strcmp ("-t", argv[i])){
			stat.role = RFC_ROLLE_TUT;
		}
		else servername = argv[i];
	}

	socket = socket_connect(port, servername);
	printf("Socket Port: %d \n", socket);
	if(socket < 0){
		fprintf(stderr, "Es konnte keine Verbindung hergestellt werden!\n");
		return EXIT_FAILURE;
	}

	//*******************Login Senden**********************
	t_net_data n_data;
	t_msg_header header;
	t_msg_login login;
	//****************** Initialisierung der Nachricht ****
	
	strncpy(login.name, user, 32);
	login.name[32] = (char)0;
	login.role = (uint8_t) stat.role;

	header.type = RFC_LOGIN;
	header.length = sizeof(t_msg_header);

	printf("Header länge ist: %d \n", header.length );

	n_data.sock = socket;
	n_data.mutex = &socket_mutex;
	n_data.len_header = sizeof(t_msg_header);
	n_data.len_exheader = sizeof(t_msg_login);
	printf("Login  header length: %d \n", n_data.len_exheader);
	n_data.len_data = (size_t) 0;

	n_data.header = (void *) &header;
	n_data.exheader = (void *) &login;
	n_data.data = NULL;

	/* Nachricht senden */
	ret = net_send(&n_data);
	if(ret < 0)
	{
		fprintf(stderr, "FEHLER: Login Nachricht konnte nicht gesendet werden!\n");
		close(socket);
		return EXIT_FAILURE;
	}
	else{
		printf("Login Nachricht wurde gesendet \n");
	}

/******************** Auf Login Reply warten **************************/
	/*FD_ZERO(&socket_fd);
	FD_SET(socket, &socket_fd);

	//////Socket+1 ???????????????
	ret = select((socket+1), &socket_fd, NULL, NULL, NULL);
	if(FD_ISSET(socket, &socket_fd))
	{
		ret = net_recv(&n_data);
		if(ret < 0)
		{
			fprintf(stderr, "Error receiving Login_Reply Leaving...\n");
			close(socket);
			return EXIT_FAILURE;
		}
		struct msg_header * _header;
		struct msg_status * _login_rep;

		_header = (struct msg_header *) n_data.header;
		if(_header->type != RFC_STATUS)
		{
			fprintf(stderr, "Protocol mismatch! Server send no Login Reply\nLeaving...\n");
			close(socket);
			return EXIT_FAILURE;
		}
		_login_rep = (struct msg_status *) n_data.exheader;*/
		/*switch(_login_rep->code)
		{
		case VT_RFC_ERROR_PERMISSION_D:
			fprintf(stderr, "Permission Denied\nLeaving...\n");
			close(vt_socket);
			return EXIT_FAILURE;
		case VT_RFC_ERROR_NAME_EXISTS:
			fprintf(stderr, "Name already in use\nLeaving...\n");
			close(vt_socket);
			return EXIT_FAILURE;
		case VT_RFC_ERROR_MAX_CLIENTS_REACHED:
			fprintf(stderr, "Max Clients Reached\nLeaving...\n");
			close(vt_socket);
			return EXIT_FAILURE;
		default:
			break;
		}
		stat.client_id = _login_rep->client_id;
		stat.role = _login_rep->role;
		printf("Login successfully\nLoged in with clientID: %d, ", stat.client_id);
		switch(stat.role)
		{
		case VT_RFC_ROLE_DOZ:
		case VT_RFC_ROLE_DOZW:
			printf("as Dozent\n");
			break;
		case VT_RFC_ROLE_STUD:
		case VT_RFC_ROLE_NDEF:
			printf("as Student\n");
			break;
		case VT_RFC_ROLE_TUT:
			printf("as Tutor\n");
			break;
		}
	}
	else /* FD nicht gesetzt */
	/*{
		fprintf(stderr, "Unexpected runtime error: Code 001\nLeaving...\n");
		close(vt_socket);
		return EXIT_FAILURE;
	}*/
/*********************************************/


	while(ret!=0){
		ret = pthread_create(&com_thread,NULL,cThread,NULL);
		printf("%d \n", ret );
	}
	ret = -1;
	while(ret!=0){
		ret = pthread_create(&list_thread,NULL,listThread,NULL);
		printf("%d \n", ret );
	}
	ret =-1;
	while(ret!=0){
		ret = pthread_create(&live_thread,NULL,liveThread,NULL);
		printf("%d \n", ret );
	}

	/* GTK threading aktivieren */
	g_thread_init(NULL);
	gdk_threads_init();


	/* GUI einrichten */
	gtk_init (&argc, &argv);
	create_window ();
	gtk_widget_show_all (GTK_WIDGET(window));

	gdk_threads_enter();
	updateGUIstate();

	/* GTK Hauptprogramm ausführen */
	gtk_main();
	gdk_threads_leave();
	
	/* cleanup here */
	

	/* Hauptprogramm verlassen */
	return 0;
}
//}

/*Help Funktion for CLI*/
static void printHelp(){
	printf("\nVirtuelle Tafel - Client\n");
	printf("------------------------\n");
	printf("Useage: %s [serveraddress] [options] [role] \n", app_name);
	printf("Example: \"%s localhost -p 59999 -n user -d\"\nConnects to localhost on port 59999 as Dozent\n", app_name);
	printf("\n");
	printf("Options\n--------------------------------------------------------\n");
	printf("-p [portnumber]\t  connect to server on Port [portnumber]\n");
	printf("-n [your name] \t  sets your name to [your name]\n");
	printf("\n");
	printf("Roles\n--------------------------------------------------------\n");
	printf("-s\tStudent\n");
	printf("-d\tDozent\n");
	printf("-t\tTutor\n");
	printf("\n");

	return;
}

