/*
 * $Id: networktransfer.c 86 2011-01-27 20:18:45Z merkelt $
 */

#include "networktransfer.h"

static int  __send(int socket, void * data, size_t len);
static int  __recv(int socket, void * data, size_t len);
static void __hton(tp_net_data s);
static int __drop(int sock, size_t n);

/**********************************************************/
/*
 * Funktion um ins Netwerk zu schreiben
 */
int net_send(tp_net_data s)
{
	int ret = 1;
	int data_to_send = 0; /* speichert den Inhalt der gesamten gesendeten Daten */

	/* Network Byte Order */
	//__hton(s);



	/* Wenn Mutex vorhanden, dann sperren */
	if(s->mutex != NULL)
	{
		ret = pthread_mutex_lock(s->mutex);
	}
	/* Wenn ein Fehler beim sperren aufgetreten ist mit -1 die Funktion beenden */
	if(ret < 0)
	{
		perror("phtread");
		return -4; /* Fehler beim Sperren des Mutex */
	}

	/************************ Senden Haeder ***************************/
	ret = __send(s->sock, s->header, s->len_header);
	printf("DEBUG: s->len_header: %i", s->len_header);
	if(ret <= 0)
	{
		pthread_mutex_unlock(s->mutex);
		return -1; /* Fehler beim Senden in Stufe 1*/
	}
	data_to_send = ret;

	/************************ Senden des Typ-Headers + Daten ***************************/
	if(s->data != NULL)
	{
		ret = __send(s->sock, s->data, s->len_data);
		if(ret <= 0)
		{
			pthread_mutex_unlock(s->mutex);
			return -2; /* Fehler beim Senden in Stufe 2 */
		}
		data_to_send += ret;
	}

	/* wenn Mutex gesetzt war wieder frei geben */
	if(s->mutex != NULL)
	{
		ret = pthread_mutex_unlock(s->mutex);
		if(ret < 0)
		{
			perror("phtread");
			return -5; /* Fehler beim freigeben des Mutex */
		}
	}

	return data_to_send;
}

/************************************************************/
int net_recv(tp_net_data s)
{
	int ret  = 0;
	int data_receive = 0;

	/************* Deklaration aller evt. benötigten Strukturen *****************/
	struct msg_header* header;
	struct msg_login * login;
	struct msg_status * status;
	struct msg_board_content * board;
	struct msg_write_request * req;
	struct msg_write_requested* write_req;
	//struct msg_error* error;


	/** Data Buffer **/
	char * data_buf;

	/*********************** initialisierung der Sende-Struktur ****************************/
	s->len_header = 0;

	s->header = NULL;

	/********************** Header auslesen **************************/
	header = (struct msg_header *) malloc(sizeof(t_msg_header));
	ret = __recv(s->sock, (void *) &header->type, sizeof(uint8_t));

	printf("DEBUG: s->sock %i\n", s->sock); 

	printf("DEBUG: header->type: %i\n", header->type);
	printf("DEBUG: ret: %i\n", ret);

	/* prüfen ob Fehler beim empfangen aufgetreten sind */
	if(ret < 0)
	{
		return -1; /* Fehler beim Lesen des Headers */
	}
	else if(ret == 0) /* Client hat die Verbindung getrennt */
	{
		return 0;
	}
	else if(ret < 1)
	{
		return -1;
	}

	ret = __recv(s->sock, (void *) &header->length, sizeof(uint16_t));

	printf("DEBUG: header->length: %i\n", header->length);
	printf("DEBUG: ret: %i\n", ret);
	
	/* prüfen ob Fehler beim empfangen aufgetreten sind */
	if(ret < 0)
	{
		return -1; /* Fehler beim lesen der Länge */
	}
	else if(ret == 0) /* Client hat die Verbindung getrennt */
	{
		return 0;
	}
	else if(ret < 2)
	{
		return -1;
	}

	printf("Header Length: %i\n", header->length);

	/* Network Byte Order to Host Byte Order */
	header->length = ntohs(header->length);

	data_receive += ret;
	s->header = (void *) header;
	s->len_header = ret;


	/*Typ der gesendeten Nachricht püfen*/
	switch(header->type)
	{
	/************************** Login ******************************/
	case RFC_LOGIN:
		/* Prüfe ob Nachricht mindestens Länge für Rolle + 1 Zeichen Name hat */
		if(header->length < 2)
		{
			printf("DEBUG: %i, %i\n", header->length, sizeof(t_msg_login));
			return -5; //länge zu kurz
		}

		/* Empfange die Daten */
		login = (struct msg_login *) malloc(sizeof(t_msg_login));
		ret = __recv(s->sock, (void * ) &login->role, sizeof(uint8_t));

		if(ret < 0)
		{
			s->data = NULL;
			return -2; /* Fehler beim Lesen des Typs */
		}
		else if(ret == 0) /* Client hat die Verbindung getrennt */
		{
			return 0;
		}

		ret = __recv(s->sock, (void * ) &login->name, header->length - 1);

		if(ret < 0)
		{
			s->data = NULL;
			return -6; /* Fehler beim Lesen in Stufe 2 */
		}
		else if(ret == 0) /* Client hat die Verbindung getrennt */
		{
			return 0;
		}
// 		else if(ret < 4)
// 		{
// 			return -7;
// 		}

		data_receive += ret;
		s->data = (void *) login;
		s->len_data = ret;

		/* Beseitige die Überhängenden Bytes */
		ret = (size_t) __drop(s->sock, (header->length - sizeof(t_msg_login)));

		if(ret < 0)
		{
			return -2;
		}else
		{
			break;
		}

	/************************ Status Nachricht **************************/
	case RFC_STATUS:
		/* Prüfe ob Nachricht die entsprechende Länge hat */
		if(header->length < sizeof(t_msg_status))
		{
			return -2;
		}

		status = (struct msg_status *) malloc(sizeof(t_msg_status));
		ret = __recv(s->sock, (void * ) status, sizeof(t_msg_status));
		if(ret < 0)
		{
			s->data = NULL;
			return -2; /* Fehler beim Lesen in Stufe 2 */
		}
		else if(ret == 0) /* Client hat die Verbindung getrennt */
		{
			return 0;
		}
		data_receive += ret;
		s->data = (void *) status;
		s->len_data = ret;

		ret = __drop(s->sock, (header->length - sizeof(t_msg_status)));
		if(ret < 0)
		{
			return -2;
		}else
		{
			break;
		}

	/************************* Tafel Update ***************************/
	case RFC_BB_CONTENT:

		if((size_t)header->length < sizeof(t_msg_board_content))
		{
			return -2;
		}

		board = (struct msg_board_content *) malloc(sizeof(t_msg_board_content));
		ret = __recv(s->sock, (void * ) board, sizeof(t_msg_board_content));
		if(ret < 0)
		{
			s->data = NULL;
			return -2; /* Fehler beim Lesen in Stufe 2 */
		}
		else if(ret == 0) /* Client hat die Verbindung getrennt */
		{
			return 0;
		}

		data_receive += ret;
		board->length = ntohs(board->length);
		board->board_recv = ntohl(board->board_recv);
		s->data = (void *) board;
		s->len_data = ret;

		/* Leere Tafel abfangen */
		if(board->length == 0)
		{
			__drop(s->sock, (header->length - sizeof(t_msg_board_content)));

			s->len_data = 0;
			s->data = NULL;
			return data_receive;
		}

		/* Prüfe ist die Length kleiner VT_MAX_PACKET_SIZE */
		if(header->length < MAX_PACKET_SIZE )
		{
			data_buf = (char *) malloc(header->length - sizeof(t_msg_board_content));
			if(data_buf == NULL)
			{
				return -5; /* Kein Speicher mehr vorhanden */
			}

			ret = __recv(s->sock, (void *) data_buf, header->length - sizeof(t_msg_board_content));
			if(ret < 0)
			{
				return -3; /* Fehler beim Lesen im Level 3 */
			}
			else if(ret == 0) /* Client hat die Verbindung getrennt */
			{
				return 0;
			}
			data_receive += ret;
			s->len_data = ret;
			s->data = (void *) data_buf;
		}
		/* Nur bis MAX_PACKET_SIZE in den Speicher lesen, den Rest verwerfen */
		else
		{
			char delete_rest; /* Mit diesem Buffer werden die Überstehenden Bytes verworfen */
			int i;
			data_buf = (char *) malloc(MAX_PACKET_SIZE - sizeof(t_msg_board_content));
			ret = __recv(s->sock, (void *) data_buf, MAX_PACKET_SIZE - sizeof(t_msg_board_content));
			if(ret < 0)
			{
				return -3;
			}
			data_receive += ret;
			s->len_data = ret;
			s->data = (void *) data_buf;

			/* Rest verwerfen */
			for(i = 0; i < (header->length - s->len_data); i++)
			{
				ret = __recv(s->sock, &delete_rest, sizeof(delete_rest));
			}
		}
		break;

	/************************* Write Request ***************************/
	case RFC_WRITE_REQUEST:
		/* Prüfe ob Nachricht die entsprechende Länge hat */
		if(header->length < sizeof(t_msg_write_request))
		{
			return -2;
		}

		req = (struct msg_write_request *) malloc(sizeof(t_msg_write_request));
		ret = __recv(s->sock, (void *) req, sizeof(t_msg_write_request));
		if(ret < 0)
		{
			return -2;
		}
		else if(ret == 0) /* Client hat die Verbindung getrennt */
		{
			return 0;
		}
		data_receive += ret;
		s->len_data = ret;
		s->data = (void *) req;
		__drop(s->sock, (header->length - sizeof(t_msg_write_request)));
		break;

	/************************ Role Requested **************************/
	case RFC_WRITE_REQUESTED:
		/* Prüfe ob Nachricht die entsprechende Länge hat */
		if(header->length < sizeof(t_msg_write_requested))
		{
			return -2;
		}

		write_req = (struct msg_write_requested *) malloc(sizeof(t_msg_write_requested));
		ret = __recv(s->sock, (void *) write_req, sizeof(t_msg_write_requested));
		if(ret < 0)
		{
			return -2;
		}
		else if(ret == 0) /* Client hat die Verbindung getrennt */
		{
			return 0;
		}
		data_receive += ret;
		write_req->client_id = ntohs(write_req->client_id);
		s->len_data = ret;
		s->data = (void *) write_req;
		ret = __drop(s->sock, (header->length - sizeof(t_msg_write_requested)));

		if(ret < 0)
		{
			return -2;
		}
		break;

	/*********************** Shutdown Meldung *************************/
	case RFC_SHUTDOWN:
		/* Níchts tun, da nur Header */
		__drop(s->sock, header->length);
		break;

	/*********************** Unbekannter Type *************************/
	default:
		return -4; /* Fehler: Unbekannter Typ */
		break;
	}
	return data_receive;
}

/************************************************************/

/* Wandelt werte in die network-byte-order */
static void __hton(tp_net_data s)
{
	struct msg_header *header;
	struct msg_status *status;
	struct msg_board_content *board;

	header = (struct msg_header *) s->header;
	if(header->type == RFC_STATUS)
	{
		status = (struct msg_status *) s->data;
		status->client_id = htons(status->client_id);
	}
	else if(header->type == RFC_BB_CONTENT)
	{
		board = (struct msg_board_content *) s->data;
		board->length = htons(board->length);
		board->board_recv = htonl(board->board_recv);
	}

	header->length = htonl(header->length);
	header->type = htonl(header->type);
}


/***********************************************************/
/* verwirft die überhängende Daten ab */
static int __drop(int sock, size_t n)
{
	size_t ind;
	size_t loops = 0;
	size_t last = 0;
	char dev_null[16];

	if(n <= (size_t) 0)
	{
		return 0;
	}

	loops = n / 16;
	last = n % 16;

	for(ind = 0; ind < loops; ind++)
	{
		if(read(sock, dev_null, 16) <= 0)
		{
			return -1;
		}
	}

	if(last != 0)
	{
		if(read(sock, dev_null, last) <= 0)
		{
			return -2;
		}
	}
	return n;
}

/****************************************************************/

/* Interne Funktion, stellt sicher, dass auch alle Bytes eines Parts gesendet werden */
static int __send(int socket, void* data, size_t len) {
	int ret = 0;
	int sended = 0;

	while ((len - sended) != 0) {
		/* unter umständen werden nicht alle Daten auf einmal gesendet, deshalb muss write unter 	*/
		/* Umständen mehrmals aufgerufen werden um alle Bytes zu senden. 							*/
		ret = write(socket, (data + sended), len - sended);
		if (ret < 0) {
			perror("write");
			return -1;
		}
		sended += ret;
	}
	return sended;
}

/****************************************************************/
/* Interne Funktion, die sicherstellt, dass alles gelesen wird. */
static int __recv(int socket, void* data, size_t length)
{
	size_t ret = 0;


	ret = read(socket, data, length);
	printf("Length: %i\n", length);
	printf("Buffer: %s\n", data);
	if(ret < 0)
	{
		return -1;
	}
	else if(ret == 0)
	{
		return 0;
	}
	else if(ret < length)
	{
		/* nicht alle erwarteten Bytes wurden empfangen */
		return -2;
	}

	return ret;
}



