#include <pthread.h>
#include <stdio.h>
#include <sys/signal.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <limits.h>
#include <string.h>
#include <strings.h>
#include <getopt.h>
#include <gtk/gtk.h>

#include "../common/messages.h"
#include "../common/sem.h"
#include "../common/mq.h"
#include "commons.h"
#include "gui.h"
#include "client.h"
#include "command.h"
#include "live.h"
#include "listener.h"

/* globale Variable */
static char toggleRequestWrite = 1;
pthread_t gui_thread_id, command_thread_id, live_thread_id, listener_thread_id;



 /**************************************************************************************\
 * Funktion:            updateGUIstate                                                  *
 *                                                                                      *
 * Kurzbeschreibung:  	Helperfunktion                                                  *
 *                      Aktualisiert die Widgets der Gui in Abhängigkeit der globalen	*
 *                      Variablen.                                                      *
 *                                                                                      *
 * Ergebnis:            int: Erfolgsstatus                                              *
 *                                                                                      *
 * Import-Parameter:    myRole, myName, myClientID                                      *
 *                      currentBoardMode                                                *
 *                      haveWritePermission                                             *
 *                      currentBoardTimestamp                                           *
 *                                                                                      *
 * Export-Parameter:    -                                                               *
 *                                                                                      *
 \**************************************************************************************/
int updateGUIstate(void) {
	static char tmp[256];
	GdkColor tmpColor;

/*	gdk_color_parse ("#385030", &tmpColor); */
	gdk_color_parse ("#386038", &tmpColor); 
	gtk_widget_modify_base(GTK_WIDGET(textview), GTK_STATE_NORMAL, &tmpColor); /* Tafelhintergrundfarbe setzen */

	sem_P(keymng_local(KEY_GCS_SEM));
	
	/* Statusleiste */
	if(GCS.status.write_per) {
		gtk_label_set_text(statusWrite, "Tafelrechte\n[schreiben]");
	} else {
		gtk_label_set_text(statusWrite, "Tafelrechte\n[lesen]");
	}

	sprintf(tmp, "Name: \t[%s]\nRolle: \t[%s]\nClientID:\t[%i]",
			GCS.name,
			(GCS.status.role == RFC_ROLE_DOZ) ? "Dozent" : (
			GCS.status.write_per ? "Tutor" : "Student"),
			GCS.status.client_id);

	gtk_label_set_text(statusRole, tmp);

	sprintf(tmp, "Zeit der letzten Änderung\n[%i]", (int)GCS.last_update);
	gtk_label_set_text(statusVersion, tmp);

	sprintf(tmp, "Dozenten: \t[%i]\nTutoren:    \t[%i]\nStudenten: \t[%i]",
		GCS.status.doz,
		GCS.status.tut,
		GCS.status.stud);

	gtk_label_set_text(statusUsers, tmp);

	toggleRequestWrite = !GCS.status.write_per;

	/* Buttons */
	sprintf(tmp,"Schreibrecht %s", toggleRequestWrite?"anfragen":"abgeben" );
	gtk_button_set_label(GTK_BUTTON(requestWrite),tmp);

	if(GCS.status.role == RFC_ROLE_DOZ) {
		gtk_widget_set_sensitive(GTK_WIDGET(requestWrite), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(logBlackboard), FALSE);
		gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),FALSE);

		if(GCS.status.write_per) {
			gtk_widget_set_sensitive(GTK_WIDGET(removePermissions), FALSE);
			gtk_widget_set_sensitive(GTK_WIDGET(logBlackboard), TRUE);
			gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),TRUE);
		} else {
			gtk_widget_set_sensitive(GTK_WIDGET(removePermissions), TRUE);
		}
	}
	if(GCS.status.role == RFC_ROLE_STUD) {
		gtk_widget_set_sensitive(GTK_WIDGET(logBlackboard), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(removePermissions), FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(quitAll), FALSE);
		gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),FALSE);
	
		if(GCS.status.write_per) {
			gtk_widget_set_sensitive(GTK_WIDGET(logBlackboard), TRUE);
			gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),TRUE);
		}
	}
	sem_V(keymng_local(KEY_GCS_SEM));

	return 0;
}


 /**************************************************************************************\
 * Funktion:            on_application_exit                                             *
 *                                                                                      *
 * Kurzbeschreibung:    Callback                                                        *
 *                      Wird beim betätigen des X-Buttons(oben rechts) aufgerufen.      *
 *                      Beendet eventuell (Benutzerentscheidung) den Client             *
 *                                                                                      *
 * Ergebnis:            gint: FALSE "destroy" einleiten; TRUE nichts machen             *
 *                                                                                      *
 * Import-Parameter:    widget: Referenz auf das auslösende Widget                      *
 *                      event: 	Eventreferenz                                           *
 *                      daten:  Zusatzinformationen                                     *
 *                                                                                      *
 * Export-Parameter:    -                                                               *
 *                                                                                      *
 \**************************************************************************************/
gint on_application_exit(GtkWidget * widget, GdkEvent event, gpointer daten)
{
	if(popupQuestionDialog("Sind Sie sicher?","Wollen Sie das Programm wirklich beenden?"))
	{
		gtk_main_quit();
		return FALSE;  /* Event "destroy" einleiten */
	}
	return TRUE;
}


 /**************************************************************************************\
 * Funktion:                    on_button_request_write_clicked                         *
 \**************************************************************************************/
void on_button_request_write_clicked (GtkButton * button, gpointer user_data)
{
	send_write_request(toggleRequestWrite);
	toggleRequestWrite = toggleRequestWrite?0:1;
	updateGUIstate();
}


 /**************************************************************************************\
 * Funktion:                    on_button_quit_all_clicked                              *
 \**************************************************************************************/
void on_button_quit_all_clicked (GtkButton * button, gpointer user_data)
{
	sem_P(keymng_local(KEY_GCS_SEM));
	if(GCS.status.role == RFC_ROLE_DOZ) {
		if(popupQuestionDialog("Sind Sie sicher?","Wollen Sie das Programm wirklich beenden?")) {
			send_shutdown();
			gtk_main_quit();
		}
	}
	sem_V(keymng_local(KEY_GCS_SEM));
}


 /**************************************************************************************\
 * Funktion:                    on_button_quit_clicked                                  *
 \**************************************************************************************/
void on_button_quit_clicked ( GtkButton * button, gpointer user_data )
{
	if(popupQuestionDialog("Sind Sie sicher?","Wollen Sie das Programm wirklich beenden?"))
		gtk_main_quit();
}

 /**************************************************************************************\
 * Funktion:                    on_button_log_clicked                                   *
 \**************************************************************************************/
void on_button_log_clicked ( GtkButton * button, gpointer user_data )
{
	send_bb_clear();
}


 /**************************************************************************************\
 * Funktion:          on_button_change_mode_clicked                                     *
 \**************************************************************************************/
void on_button_remove_permissions_clicked ( GtkButton * button, gpointer user_data )
{
	sem_P(keymng_local(KEY_GCS_SEM));
	if(GCS.status.role == RFC_ROLE_DOZ)
		send_write_request(1);
	sem_V(keymng_local(KEY_GCS_SEM));
}


 /**************************************************************************************\
 * Funktion:           on_text_buffer_changed                                           *
 \**************************************************************************************/
void on_text_buffer_changed (GtkTextBuffer *textbuffer, gpointer user_data)
{
	sem_P(keymng_local(KEY_GCS_SEM));
	if(GCS.status.write_per) {
		GtkTextIter startIter, endIter; 
		char *tmpBoard;
		gtk_text_buffer_get_start_iter(textbuffer,&startIter);
		gtk_text_buffer_get_end_iter(textbuffer,&endIter);

		tmpBoard = gtk_text_buffer_get_text(textbuffer,&startIter,&endIter,FALSE);

		GCS.board = tmpBoard;
		sem_V(keymng_local(KEY_LIVE_SEM)); /* trigger live thread to send content */
	}
	sem_V(keymng_local(KEY_GCS_SEM));
}



void print_help(char *self) {
	printf("%s [params] [server]\n"
	       "   parameters:\n"
	       "                 -h  --help\n"
	       "                 -n  --name <Guest>\n"
	       "                 -p  --port <5000>\n"
	       "                 -s  --student\n"
	       "                 -d  --dozent\n"
	       "\n", self);
	exit(0);
}

 
 /**************************************************************************************\
 * Funktion: start                                                                       *
 \**************************************************************************************/
int main(int argc, char **argv) {
	int role = RFC_ROLE_DONTCARE;
	char *name   = "Guest";
	char *server = "localhost";
	char *port   = "50000";
	int c, ret;

	struct addrinfo *addr_info, *p;

	while(optind < argc) {
		int option_index = 0;
		static struct option long_options[] = {
			{"name",    optional_argument, 0, 'n'},
			{"port",    optional_argument, 0, 'p'},
			{"student", optional_argument, 0, 's'},
			{"dozent",  optional_argument, 0, 'd'},
			{"help",    no_argument,       0, 'h'},
			{0,0,0,0}
		};
		c = getopt_long(argc, argv, "dshn:p:", long_options, &option_index);
		if(c == -1) break;

		switch(c) {
			case '?': /* unknown parameter */
			case ':': /* missing argument */
				print_help(argv[0]);
				break;
			case 'n':
				name = strdup(optarg);
				if(!name) return -1;
				break;
			case 'p':
				port = strdup(optarg);
				break;
			case 'h':
				print_help(argv[0]);
				break;
			case 'd':
				role = RFC_ROLE_DOZ;
				break;
			case 's':
				role = RFC_ROLE_STUD;
				break;
			case '0':
				/*
				opt = (struct option *) &(long_options[option_index]);
				printf("set server to: %s=%s\n", opt->name, optarg);
				*/
				break;
			default:
				break;
		}
	}
	while(optind < argc) {
		server = argv[optind++];
	}


	signal(SIGINT, sigint_handler);
	
	GCS.board = NULL;
	bzero(&GCS.status, sizeof(t_msg_status));
	GCS.name = name;
	GCS.role = role;

	sem_V(keymng_local(KEY_GCSSOCK_SEM));
	sem_V(keymng_local(KEY_GCS_SEM));

	ret = getaddrinfo(server, port, NULL, &addr_info);
	if(ret) {
		printf("getaddrinfo: %s\n", gai_strerror(ret));
		exit(-1);
	}

	p = addr_info;

	while(p) {
		if(p->ai_socktype != SOCK_STREAM) { /* we only care about TCP */
			p = p->ai_next;
			continue;
		}
		int s = socket(p->ai_family, p->ai_socktype, 0);
		if(s == -1) {
			perror("socket");
			exit(-1);
		}

		if(connect(s, p->ai_addr, p->ai_addrlen) == 0) {

			GCS.s = s;

			ret = pthread_create(&gui_thread_id, NULL, &gui_thread, NULL);
			if(ret) {
				printf("Failed to start GUI-Thread\n");
				exit(0);
			}

			sleep(1); /* mutex me please */

			ret = pthread_create(&command_thread_id, NULL, &command_thread, NULL);
			if(ret) {
				printf("Failed to start Command-Thread\n");
				exit(0);
			}

			ret = pthread_create(&live_thread_id, NULL, &live_thread, NULL);
			if(ret) {
				printf("Failed to start Live-Thread\n");
				exit(0);
			}

			ret = pthread_create(&listener_thread_id, NULL, &listener_thread, NULL);
			if(ret) {
				printf("Failed to start Listener-Thread\n");
				exit(0);
			}

			pthread_join(command_thread_id, NULL);
			pthread_join(live_thread_id, NULL);
			pthread_join(listener_thread_id, NULL);
			pthread_join(gui_thread_id, NULL);
			return 0;
		}
		close(s);
		p = p->ai_next;
	}
	printf("Could not connect to server :/\n");
	raise(SIGINT);
	freeaddrinfo(addr_info);
	return 0;
}

void *gui_thread(void *data) {
	/* GTK threading aktivieren */
	g_thread_init(NULL);
	gdk_threads_init();


	/* GUI einrichten */
	gtk_init (0, NULL); /* fucking gtk takes "name"?! */
	create_window ();
	gtk_widget_show_all (GTK_WIDGET(window));

	gdk_threads_enter();
	updateGUIstate();

	/* GTK Hauptprogramm ausführen */
	gtk_main();
	gdk_threads_leave();

	printf("GUI Exit, cleanup mission started...\n");
	raise(SIGINT);
	return NULL;
}

void sigint_handler(int sig) {
	signal(sig, SIG_IGN);
	
	printf("Shutting down... ");

	/* fuck off, doing things aem random things break all */
	pthread_cancel(command_thread_id);
	sem_remove(keymng_local(KEY_LIVE_SEM));
	sem_remove(keymng_local(KEY_GCS_SEM));
	sem_remove(keymng_local(KEY_GCSSOCK_SEM));
	pthread_cancel(live_thread_id);
	pthread_cancel(listener_thread_id);
	pthread_cancel(gui_thread_id);

	message_queue_remove(keymng_local(KEY_COMMAND_MQ));

	printf("OK\n");
	
	exit(0);
}
