#include <stdlib.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <string.h>

#include "../common/mq.h"
#include "../common/sem.h"
#include "command.h"
#include "client.h"

void *command_thread(void *data) {
	MSG msg;
	uint16_t len;

	sem_P(keymng_local(KEY_GCS_SEM));
	send_login(GCS.s, GCS.name, GCS.role);
	sem_V(keymng_local(KEY_GCS_SEM));

	while(1) {
		msg = message_queue_receive_command();
		memcpy(&len, msg.text+1, sizeof(uint16_t));
		if(ntohs(len) > 1277) continue; /* sorry, but the message was lost in the mq -> don't send */
		sem_P(keymng_local(KEY_GCSSOCK_SEM));
		send(GCS.s, msg.text, ntohs(len) + sizeof(t_msg_header), 0);
		sem_V(keymng_local(KEY_GCSSOCK_SEM));
	}

	return NULL;
}


void send_login(int s, char *name, int role) {
	char msg[MAX_MSG];
	t_msg_header hdr;

	bzero(msg, sizeof(msg));
	
	hdr.type = RFC_LOGIN;
	hdr.length = htons(strlen(name) + 1);
	
	memcpy(msg,
	       &hdr, sizeof(hdr));
	memcpy(msg + sizeof(hdr),
	       &role, sizeof(uint8_t));
	memcpy(msg + sizeof(hdr) + sizeof(uint8_t),
	       name, strlen(name));

	message_queue_send_command(msg);
}

void send_request_reply(int s, uint8_t write_per, uint16_t client_id) {
	char msg[MAX_MSG];
	t_msg_header hdr;
	uint16_t cid = htons(client_id);

	bzero(msg, sizeof(msg));
	
	hdr.type = RFC_REQUEST_REPLY;
	hdr.length = htons(sizeof(uint16_t) + sizeof(uint8_t));
	
	memcpy(msg,
	       &hdr, sizeof(hdr));
	memcpy(msg + sizeof(hdr),
	       &write_per, sizeof(uint8_t));
	memcpy(msg + sizeof(hdr) + sizeof(uint8_t),
	       &cid, sizeof(uint16_t));

	message_queue_send_command(msg);
}

void send_bb_clear() {
	char msg[MAX_MSG];
	t_msg_header hdr;
	
	bzero(msg, sizeof(msg));
	
	hdr.type = RFC_BB_CLEAR;
	hdr.length = 0;

	memcpy(msg, &hdr, sizeof(hdr));

	message_queue_send_command(msg);
}

void send_shutdown() {
	char msg[MAX_MSG];
	t_msg_header hdr;
	
	bzero(msg, sizeof(msg));
	
	hdr.type = RFC_SHUTDOWN;
	hdr.length = 0;

	memcpy(msg, &hdr, sizeof(hdr));

	message_queue_send_command(msg);
}

void send_write_request(uint8_t write_per) {
	char msg[MAX_MSG];
	t_msg_header hdr;

	bzero(msg, sizeof(msg));

	hdr.type = RFC_WRITE_REQUEST;
	hdr.length = htons(sizeof(uint8_t));
	
	memcpy(msg,
		   &hdr, sizeof(hdr));
	memcpy(msg + sizeof(hdr),
		   &write_per, sizeof(uint8_t));

	message_queue_send_command(msg);
}
