#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <signal.h>
#include "../common/sem.h"
#include "listener.h"
#include "client.h"
#include "gui.h"
#include "command.h"

void *listener_thread(void *data) {
	t_msg_header hdr;
	int parse_ok = 0;
	int ret;

	printf("Connected!\n");


	while( 1 ) {
		ret = recv(GCS.s, &hdr, sizeof(hdr), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(hdr)) {
			raise(SIGINT);
		}

		hdr.length = ntohs(hdr.length);
		parse_ok = parse_msg(&hdr);
		if(parse_ok == 1) {
			raise(SIGINT);
		}
	}
	return 0;
}


int parse_msg(t_msg_header *hdr) {
	size_t ret = 0;

	//printf("msg: %x\n", hdr->type);
		
	if(hdr->type == RFC_STATUS) {
		sem_P(keymng_local(KEY_GCS_SEM));
		ret = recv(GCS.s, &GCS.status, sizeof(t_msg_status), MSG_WAITALL);
		sem_V(keymng_local(KEY_GCS_SEM));
		if(ret == 0 || ret < sizeof(t_msg_status)) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}
		
		sem_P(keymng_local(KEY_GCS_SEM));
		GCS.status.stud      = ntohs(GCS.status.stud);
		GCS.status.client_id = ntohs(GCS.status.client_id);

		GCS.last_update = time(NULL);
		sem_V(keymng_local(KEY_GCS_SEM));
		
		gdk_threads_enter();
		updateGUIstate();
		gdk_threads_leave();


	} else if(hdr->type == RFC_BB_CONTENT) {
		if(hdr->length == 0) {
			set_board("", 0);
			return 0;
		}

		char *board = malloc(hdr->length + 1);
		if(!board) return ERR_OOM;

		ret = recv(GCS.s, board, hdr->length, MSG_WAITALL);
		if(ret == 0 || ret < hdr->length) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}

		board[hdr->length] = '\0';
		set_board(board, hdr->length);
		free(board);

	} else if(hdr->type == RFC_WRITE_REQUESTED) {
		int client_id;
		ret = recv(GCS.s, &client_id, sizeof (uint16_t), MSG_WAITALL);
		
		if(ret == 0 || ret < sizeof (uint16_t)) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}
		client_id = ntohs(client_id);

		char *reqName = malloc(hdr->length - 1 );
		ret = recv(GCS.s, reqName, hdr->length - 2, MSG_WAITALL);
		if(ret == 0 || ret < hdr->length - 2) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}
		reqName[hdr->length - 2] = '\0';

		GString *question = g_string_new(NULL);
		g_string_sprintf(question, "Der Benutzer %s (%d) erfragt Schreibricht. Zugriff gewähren?", reqName, client_id);
		
		free(reqName);
		
		gdk_threads_enter();
		ret = popupQuestionDialog("Schreibrecht Anfrage", question->str);
		gdk_threads_leave();

		g_string_free(question, TRUE);

		send_request_reply(GCS.s, ret, client_id);

	} else if(hdr->type == RFC_ERROR) {
		uint8_t error_code;
		char *error_message = "Fail!";
		ret = recv(GCS.s, &error_code, sizeof(uint8_t), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(uint8_t)) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}

		if(hdr->length != 1) {
			error_message = malloc(hdr->length);
			if(!error_message) return ERR_OOM;
			ret = recv(GCS.s, error_message, hdr->length - 1, MSG_WAITALL);
			if(ret == 0 || ret < hdr->length - 1) { /* disconnect or error */
				return ERR_KILL_CLIENT;
			}
			error_message[hdr->length - 1] = '\0';
		}
		gdk_threads_enter();
		popupMessage(error_message);
		gdk_threads_leave();
		
		printf("Error <code %x>\n", error_code);
		switch(error_code) {
			case RFC_ERROR_LOGIN_FAIL:
			case RFC_ERROR_FATAL:
				raise(SIGINT);
			case RFC_ERROR_WARN:
				return 0;
			default:
				/* unknown error type */
				return ERR_KILL_CLIENT;		
		}
	}

	return 0;
}

void set_board(char* black_content, int length) {

    GtkTextBuffer *buffer;

    gdk_threads_enter();

    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
    gtk_text_buffer_set_text(buffer, black_content, length);

    gdk_threads_leave();
}

