/* 
 * $Id: messages.h 71 2011-01-19 16:24:51Z merkelt $
 *
 * Sources:
 * - http://sig9.com/articles/gcc-__packed__-structures
 */
#ifndef MESSAGES_H_
#define MESSAGES_H_
/*
 * Systemincludes
 */

#include <stdint.h>
#include <pthread.h>
#include <sys/types.h>

#include "common.h"

/*
 * RFC type defines
 */
#define RFC_LOGIN             0
#define RFC_STATUS            1
#define RFC_BB_CONTENT        2
#define RFC_BB_CLEAR          3
#define RFC_SHUTDOWN          4
#define RFC_WRITE_REQUEST     5
#define RFC_WRITE_REQUESTED   6
#define RFC_REQUEST_REPLY     7
#define RFC_ERROR           255

/*
 * RFC role defines
 */

#define RFC_ROLE_DONTCARE 0
#define RFC_ROLE_STUD     1
#define RFC_ROLE_DOZ      2
#define RFC_ROLE_TUT      4


/*
 * RFC Error Codes
 */

#define RFC_ERROR_WARN        0
#define RFC_ERROR_FATAL       1
#define RFC_ERROR_LOGIN_FAIL 10


/*
 * Maximal Packet size
 */
#define MAX_PACKET_SIZE 1232 //Header (24) + Type BB_Content (8) + BlackBoard (15 Zeilen * 80 Zeichen =1200) =1232


/*
 * Network structs for sending data in the correct rfc shema
 * - https://www.elearning.hs-weingarten.de/mod/resource/view.php?id=6345
 */
struct msg_header {
	uint8_t type;    // describe the type of the message
	uint16_t length; // length of the next bytes
} __attribute__((__packed__));

typedef struct msg_header t_msg_header;


//Typ 0
struct msg_login {
	uint8_t role;          // 0 = whatever, 1 = student, 2 = dozent
	char    *name; // login name without \0 (length-1 byte)
} __attribute__((__packed__));

typedef struct msg_login t_msg_login;


//Typ 1
struct msg_status {
	uint8_t role;         // 1 = student, 2 = dozent
	uint16_t client_id;   // client identification number
	uint8_t write_per;    // 0 = no write permission, 1 = write permission
	uint8_t doz;          // count of dozents
	uint8_t tut;          // count of tutors
	uint16_t stud;        // count of students
} __attribute__((__packed__));

typedef struct msg_status t_msg_status;

//Typ 2
struct msg_board_content{
	uint16_t length;
	uint32_t board_recv;//Board-Content wird nicht in der Struktur gespeichert, es muss zuerst die größe ermittelt werden

} __attribute__((__packed__));
typedef struct msg_board_content t_msg_board_content;


//Typ 5
struct msg_write_request {
	uint8_t write_per;    // 0 = no write permission, 1 = write permission
} __attribute__((__packed__));
typedef struct msg_write_request t_msg_write_request;


//Typ 6
struct msg_write_requested {
	uint16_t client_id;   // client identification number
	char     *name;
} __attribute__((__packed__));
typedef struct msg_write_requested t_msg_write_requested;

//Typ 7
struct msg_write_reply {
	uint8_t write_per;
	uint16_t client_id;
} __attribute__((__packed__));
typedef struct msg_write_reply t_msg_write_reply;

//Typ 255
struct msg_error {
	uint8_t error_code;
} __attribute__((__packed__));
typedef struct msg_error t_msg_error;




#endif
