/* 
 * $Id: client.mod.c 97 2011-01-28 11:32:53Z merkelt $
 *
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include "../common/network.h"
#include "../common/mq.h"
#include "../common/sem.h"
#include "shm.h"
#include "clientlist.mod.h"
#include "client.mod.h"

void client_cleanup(tp_client_info ci) {
	close(ci->sock);
	if(ci->client_id > -1)
		clientlist_remove(ci->client_id);
	free(ci->name);
	free(ci);
	pthread_exit(0);
}


void *clientThread(void *data) {
	t_msg_header hdr;
	tp_client_info ci = (tp_client_info) data;
	size_t ret = 0;
	int parse_ok = 0;

	while( 1 ) {
		ret = recv(ci->sock, &hdr, sizeof(hdr), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(hdr)) {
			client_cleanup(ci);
		}

		hdr.length = ntohs(hdr.length);
		parse_ok = parse_msg(ci, &hdr);
		if(parse_ok == ERR_KILL_CLIENT) {
			client_cleanup(ci);
		}
	}
}

void send_error_msg(tp_client_info ci, uint8_t error_code, char *error_msg) {
	t_msg_header hdr;
	hdr.type = RFC_ERROR;
	hdr.length = htons(sizeof(uint8_t) + strlen(error_msg));
	
	LOG("Some error send to the client\n");
	
	send(ci->sock, &hdr, sizeof(hdr), MSG_MORE);
	send(ci->sock, &error_code, sizeof(uint8_t), MSG_MORE);
	send(ci->sock, error_msg, sizeof(char) * (strlen(error_msg)), 0);
// 	fflush((FILE*) ci->sock);
}

int parse_msg(tp_client_info ci, t_msg_header *hdr) {
	size_t ret = 0;

	/* we use if, because switch/case dosn't let us allocate variables dynamicaly */
	if(hdr->type == RFC_LOGIN) {
		int ok = 0;
		if(hdr->length < 2) {
			send_error_msg(ci, RFC_ERROR_LOGIN_FAIL, "Invalid packet length");
			return ERR_KILL_CLIENT;
		}
		t_msg_login *msg = malloc(sizeof(t_msg_login));
		if(!msg) return ERR_OOM;
		msg->name = malloc(sizeof(char) * (hdr->length));
		if(!msg->name) {
			free(msg);
			return ERR_OOM;
		}
		
		ret = recv(ci->sock, msg, sizeof(uint8_t), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(uint8_t)) { /* disconnect or error */
			free(msg->name);
			free(msg);
			return ERR_KILL_CLIENT;
		}
		ret = recv(ci->sock, msg->name, sizeof(char) * (hdr->length - 1), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(char) * (hdr->length - 1)) { /* disconnect or error */
			free(msg->name);
			free(msg);
			return ERR_KILL_CLIENT;
		}
		msg->name[hdr->length - 1] = '\0';
		ok = handle_login(ci, msg);
		free(msg);
		return ok;

	} else if(hdr->type == RFC_BB_CONTENT) {
		char *board = NULL;
		/* write permission? */
		if(!client_has_write_permission(ci, clientlist_tutcount())) {
			send_error_msg(ci, RFC_ERROR_WARN, "You don't have permission to write the board content!");
			return 0;
		}
		if(hdr->length > BOARD_LEN) {
			send_error_msg(ci, RFC_ERROR_WARN, "Your message is to long to handle by the server!");
			return 0;
		}		
		if(hdr->length > 0) {
			board = malloc(sizeof(char) * hdr->length+1);
			if(!board) return ERR_OOM;
			ret = recv(ci->sock, board, sizeof(char) * hdr->length, MSG_WAITALL);
			if(ret == 0 || ret < sizeof(char) * (hdr->length)) { /* disconnect or error */
				return ERR_KILL_CLIENT;
			}
			sem_P(keymng(KEY_SHM_SEM));
			strncpy(Board, board, hdr->length);
			Board[hdr->length+1] = '\0';
			sem_V(keymng(KEY_SHM_SEM));
			free(board);
		} else { /* zero length board content -> clear it */
			sem_P(keymng(KEY_SHM_SEM));
			shm_init(Board);
			sem_V(keymng(KEY_SHM_SEM));
		}
		sem_V(keymng(KEY_BCA_SEM));
	} else if(hdr->type == RFC_BB_CLEAR) {
		if(!client_has_write_permission(ci, clientlist_tutcount())) {
			send_error_msg(ci, RFC_ERROR_WARN, "You don't have permission to clear the board content!");
			return 0;
		}
		
		kill(ArchivePid, SIGUSR1); /* trigger archiver */
		sem_P(keymng(KEY_ARC_SEM));
		sem_P(keymng(KEY_SHM_SEM));
		shm_init(Board);
		sem_V(keymng(KEY_SHM_SEM));
		sem_V(keymng(KEY_BCA_SEM));

	} else if(hdr->type == RFC_SHUTDOWN) {
		
		if(ci->role != RFC_ROLE_DOZ) {
			send_error_msg(ci, RFC_ERROR_WARN, "You're not man enough to kill me!");
			return 0;
		}
			
		sem_P(keymng(KEY_CLIENTLIST_SEM));
		client *c  = clientlist_getroot();
		while(c != NULL) {
			send_error_msg(c->p, RFC_ERROR_WARN, "System going down for a nap!");
			c = c->next;
		}
		sem_V(keymng(KEY_CLIENTLIST_SEM));

		kill(ArchivePid, SIGUSR1); /* trigger archiver */
		raise(SIGINT);
	} else if(hdr->type == RFC_WRITE_REQUEST) {
		uint8_t writeper;
		ret = recv(ci->sock, &writeper, sizeof(uint8_t), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(uint8_t)) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}
		if( writeper > 1 ) {
			send_error_msg(ci, RFC_ERROR_WARN, "Uuuhm, invalid write-permission!");
			return ERR_IGNORE;
		}
		
		if(ci->role == RFC_ROLE_STUD && writeper) {/* anfordern */
			/* send msg to doz with question */
			send_write_requested(ci);
		} else if(ci->role & RFC_ROLE_TUT && !writeper) { /* abgeben */
			ci->role = RFC_ROLE_STUD;
			sem_V(keymng(KEY_BCA_SEM));
		} else if (ci->role == RFC_ROLE_DOZ && writeper) { /* entziehen */
			tp_client_info cit = clientlist_getbyrole(RFC_ROLE_TUT);
			if(!cit) return ERR_IGNORE;

			cit->role = RFC_ROLE_STUD;
			sem_V(keymng(KEY_BCA_SEM));
		} else {
			send_error_msg(ci, RFC_ERROR_WARN, "Strange combination of role / writepermission!");
			return ERR_IGNORE;
		}

		LOG("Write Request\n");
	} else if(hdr->type == RFC_REQUEST_REPLY) {
		uint8_t write_per;
		ret = recv(ci->sock, &write_per, sizeof(uint8_t), MSG_WAITALL);
		if(ret == 0 || ret < sizeof(uint8_t)) { /* disconnect or error */
			return ERR_KILL_CLIENT;
		}
		uint16_t client_id;
		ret = recv(ci->sock, &client_id, sizeof(uint16_t), MSG_WAITALL);
		client_id = htons(client_id);


		if(ci->role != RFC_ROLE_DOZ) {
			send_error_msg(ci, RFC_ERROR_WARN, "This message could only be received from an instructor!");
			return 0;
		}

		tp_client_info ti = clientlist_getbyrole(RFC_ROLE_TUT);
		if(ti) {
			ti->role = RFC_ROLE_STUD;
		}

		client *cit = clientlist_getbycid(client_id);
		if( write_per == 1 ) {
			cit->p->role |= RFC_ROLE_TUT;
		}
		sem_V(keymng(KEY_BCA_SEM));
		LOG("Request Reply\n");
	} else {
		send_error_msg(ci, RFC_ERROR_WARN, "Invalid message recieved. Thanks!");
	}

	return 0;
}

int client_has_write_permission(tp_client_info ci, int tutcount) {
	return (ci->role == RFC_ROLE_DOZ && tutcount < 1) ||
	       (ci->role  & RFC_ROLE_TUT);
}

void send_write_requested(tp_client_info ci) {
	t_msg_header hdr;
	uint16_t client_id = htons(ci->client_id);

	hdr.type   = RFC_WRITE_REQUESTED;
	hdr.length = htons(sizeof(uint16_t) + strlen(ci->name));

	tp_client_info cit = clientlist_getbyrole(RFC_ROLE_DOZ);
	if(!cit) return;

	send(cit->sock, &hdr, sizeof(hdr), MSG_MORE);
	send(cit->sock, &client_id, sizeof(uint16_t), MSG_MORE);
	send(cit->sock, ci->name, sizeof(char) * (strlen(ci->name)), 0);
}

int handle_login(tp_client_info ci, t_msg_login *msg) {
	switch(msg->role) {
		case RFC_ROLE_DONTCARE:
			if(clientlist_dozcount() < 1) {
				ci->role = RFC_ROLE_DOZ;
				break;
			}
		case RFC_ROLE_STUD:
			ci->role = RFC_ROLE_STUD;
			break;
		case RFC_ROLE_DOZ:
			if(clientlist_dozcount()) {
				ci->role = RFC_ROLE_STUD;
			} else {
				ci->role = RFC_ROLE_DOZ;
			}
			break;
		default:
			send_error_msg(ci, RFC_ERROR_LOGIN_FAIL, "Invalid Role");
			return ERR_KILL_CLIENT;
	}

	ci->name = msg->name;

	if(!clientlist_add(ci)) {
		send_error_msg(ci, RFC_ERROR_FATAL, "Server Fail");
		return ERR_KILL_CLIENT;
	}
	LOG("Client successful added to the client list\n");

	return 0;
}
