/*
 * $Id: clientlist.mod.c 86 2011-01-27 20:18:45Z merkelt $
 */

#include <stdlib.h>
#include <stdio.h>
#include "clientlist.mod.h"
#include "../common/sem.h"
#include "../common/mq.h"
#include "../common/messages.h"

client* rootnode_ptr = NULL;
uint16_t client_id_counter = 0;

tp_client_info clientlist_add(tp_client_info ci) {
	client *c;
	c = (client* ) malloc(sizeof(client));
	if(c == 0) {
		LOG("Error client malloc!\n");
		return NULL;
	}

	sem_P(keymng(KEY_CLIENTLIST_SEM));

	c->p = ci;
	ci->client_id = ++client_id_counter;
	c->next = rootnode_ptr;
	rootnode_ptr = c;

	sem_V(keymng(KEY_CLIENTLIST_SEM));

	/* status an alle über bc-agent! */
	sem_V(keymng(KEY_BCA_SEM));
	return ci;
}

void clientlist_remove(uint16_t cid) {
	client *tmp_ptr1, *tmp_ptr2;
	tmp_ptr1 = tmp_ptr2 = rootnode_ptr;

	if(rootnode_ptr == NULL) return;

	tmp_ptr1 = clientlist_getbycid(cid);
	if(tmp_ptr1 == NULL) {
		LOG("Error clientlist_remove with invalid client_id!\n");
		return;
	}

	sem_P(keymng(KEY_CLIENTLIST_SEM));
	if(tmp_ptr1 == rootnode_ptr) {
		rootnode_ptr = tmp_ptr1->next;
		free(tmp_ptr1);
	} else {
		while(tmp_ptr2->next != tmp_ptr1)
			tmp_ptr2 = tmp_ptr2->next;
		tmp_ptr2->next = tmp_ptr1->next;
		free(tmp_ptr1);
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	/* status an alle über bc-agent! */
	sem_V(keymng(KEY_BCA_SEM));
}


void clientlist_removeall() {
	client *tmp_ptr1, *tmp_ptr2;

	if(rootnode_ptr == NULL) return;
	
	sem_P(keymng(KEY_CLIENTLIST_SEM));
	tmp_ptr1 = rootnode_ptr;	

	while(tmp_ptr1 != NULL) {
		tmp_ptr2 = tmp_ptr1;
		tmp_ptr1 = tmp_ptr1->next;
		free(tmp_ptr2);
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
}

client* clientlist_getbycid(uint16_t cid) {
	client *tmp_c;
	
	if(rootnode_ptr == NULL) return NULL;
	sem_P(keymng(KEY_CLIENTLIST_SEM));
	tmp_c = rootnode_ptr;
	while(tmp_c && tmp_c->p->client_id != cid) {
		if(tmp_c == NULL) {
			sem_V(keymng(KEY_CLIENTLIST_SEM));
			return NULL;
		}
		tmp_c = tmp_c->next;
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	return tmp_c;
}


client* clientlist_getbytid(pthread_t tid) {
	client *tmp_c;
	
	if(rootnode_ptr == NULL) return NULL;

	sem_P(keymng(KEY_CLIENTLIST_SEM));
	tmp_c = rootnode_ptr;
	while(tmp_c->p->thread != tid) {
		if(tmp_c == NULL) {
			sem_V(keymng(KEY_CLIENTLIST_SEM));
			return NULL;
		}
		tmp_c = tmp_c->next;
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	return tmp_c;
}


client* clientlist_getroot() {
	return rootnode_ptr;
}



int clientlist_studcount() {
	client *c;
	int i = 0;
	sem_P(keymng(KEY_CLIENTLIST_SEM));
	c = clientlist_getroot();
	while(c != NULL) {
		if(c->p->role == RFC_ROLE_STUD) i++;
		c = c->next;
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	return i;
}

int clientlist_dozcount() {
	client *c;
	int i = 0;
	sem_P(keymng(KEY_CLIENTLIST_SEM));
	c = clientlist_getroot();
	while(c != NULL) {
		if(c->p->role == RFC_ROLE_DOZ) i++;
		c = c->next;
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	return i;
}

int clientlist_tutcount() {
	client *c;
	int i = 0;
	sem_P(keymng(KEY_CLIENTLIST_SEM));
	c = clientlist_getroot();
	while(c != NULL) {
		if(c->p->role & RFC_ROLE_TUT) i++;
		c = c->next;
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	return i;
}

tp_client_info clientlist_getbyrole(uint8_t role) {
	client *c;
	sem_P(keymng(KEY_CLIENTLIST_SEM));
	c = clientlist_getroot();
	while(c != NULL) {
		if(c->p->role & role) {
			sem_V(keymng(KEY_CLIENTLIST_SEM));
			return c->p;
		}
		c = c->next;
	}
	sem_V(keymng(KEY_CLIENTLIST_SEM));
	return NULL;
}
