/*
 * $Id: server.c 96 2011-01-28 09:43:20Z merkelt $
 *
 * First server program, includes the main and initialize all
 * the other threads - logger, archiver and so on...
 */

#include <signal.h>
#include "server.h"
#include "broadcastagent.mod.h"
#include "../common/sem.h"
#include "shm.h"
#include "../common/mq.h"
#include "logger.h"
#include "archive.h"
#include "../common/common.h"

/*
 * Global Thread IDs
 */
pthread_t loginThread_id;
pthread_t bcaThread_id;

int loggerPid;

void sigint_handler(int sig) {
	signal(sig, SIG_IGN);
	removePID();
	
	sem_remove(keymng(KEY_BCA_SEM));
	sem_remove(keymng(KEY_ARC_SEM));
	sem_remove(keymng(KEY_CLIENTLIST_SEM));
	sem_remove(keymng(KEY_SHM_SEM));
	message_queue_remove(keymng(KEY_LOGGER_MQ));

	/* close sockets, join threads, cleanup stuff */
	kill(loggerPid, SIGKILL);
	kill(ArchivePid, SIGKILL);

	shm_remove(shmget(keymng(KEY_SHM), 0, 0));

	printf("\nGood Night!\n");
	exit(0);
}

int main( int argc, char **argv ) {
	// Variables
	int pid = 0;               // current pid
	int port = 50000;          // default port
	int i = 0;
	int ret;
	int test_interval = 0;     // debug interval

	// Check argument list
	for( i = 1; i < argc; ++i ) {
		if( !strcmp("-p", argv[i]) ) {
			if( ++i < argc ) {
				port = atoi( argv[i] );
			} else {
				printHelp(argv[0]);
				return -1;
			}
		}
		else if( !strcmp("-d", argv[i]) ) {
			if( ++i < argc ) {
				test_interval = atoi(argv[i]);
			} else {
				printHelp(argv[0]);
				return -1;
			}
		}
		else {
			printHelp(argv[0]);
			return -1;
		}
	}

	// Check if process already running
	pid = checkPID();
	if( pid > 0 ) {
		printf("Server already running. Check pid (%d) file.\n", pid);
		return -1;
	} else {
		printf("Server run\n");
		createPID();
	}

	/*
	 * Start process communication for all subprocesses
	 */

	// Create Shared Memory
	Board = (char *) shm_attach(shm_create());
	if(!Board) {
		LOG("Failed to allocate shmmmnomnomnom\n");
		raise(SIGINT);
	}
	shm_init(Board);

	// Fork Logger
	loggerPid = fork();
	if( loggerPid == 0 ) { 
		exit(logger());
	}

	// Fork Archive
	ArchivePid = fork();
	if( ArchivePid == 0 ) {
		exit(archive(test_interval));
	}

	// Install signal handler
	signal(SIGINT, sigint_handler);

	// free julian assange
	sem_V(keymng(KEY_SHM_SEM));
	sem_V(keymng(KEY_CLIENTLIST_SEM));

	/*
	 * Broadcast Agent
	 */
	ret = pthread_create(&bcaThread_id, NULL, &bcaThread, NULL);
	if(ret) {
		LOG("Failed to start Broadcast Agent Thread\n");
		raise(SIGINT);
	}
	pthread_detach(bcaThread_id);

	/*
	 * Login Thread
	 */
	LOG("Starting login thread...\n");
	ret = pthread_create(&loginThread_id, NULL, &loginThread, (void *)&port);
	if( ret != 0 ) {
		perror("pthread_create");
		LOG("Login thread failed to start!\n");
		// cleanup
		raise(SIGINT);
	}
	LOG("Login thread started\n");

	// Waiting for loginThread
	pthread_join(loginThread_id, NULL);

	return 0;
}




/*
 * Help
 */
static void printHelp(char *self) {
	printf("%s [params]\n"
		   "   parameters:\n"
		   "                 -p <port number>\n"
		   "                 -d <test interval>\n"
		   "\n", self);
}

/*
 * All the stuff for pid and running process checking
 */
static void createPID() {
	// getpid()
	FILE *pidFile = fopen("server.pid", "w");
	fprintf(pidFile, "%d", getpid());
	fclose(pidFile);
}

static int checkPID() {
	char puffer[32];
	FILE *pidFile = fopen("server.pid", "r");

	if( pidFile != NULL ) {
		fgets(puffer, 80, pidFile);
		return atoi(puffer);
	} else {
		return -1;
	}
}

static int removePID() {
	return unlink("server.pid");
}
